/***************************************************************************
                          dialdialogimp.cpp  -  description
                             -------------------
    begin                : Wed Dec 26 2001
    copyright            : (C) 2001 by Robin Atwood
    email                : robin.atwood@attglobal.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "logviewdialogimp.h"
#include "dialdialogimp.h"
#include "recursiveitemfetchjob.h"
//Added by qt3to4:
#include <QKeyEvent>
#include <QDebug>

using namespace KABC;
#include <KABC/VCardConverter>
#include <KABC/PhoneNumber>

DialDialog::DialDialog( QWidget* parent, Qt::WFlags fl ) :
		Ui::DialDialog()															
{
	dialling = false;
	modem = 0;
	locked = 0;
  redialCount = 0;
	intvlLeft = 0;

	setupUi( this );
	dialDialButton->setFocus();
	dialDialButton->setIcon(KIcon("kall"));
	dialMenuButton->setIcon(KIcon("view-list-tree"));
	dialCancelButton->setIcon(KIcon("dialog-cancel"));

	QMenu *menu = new QMenu( this );
	viewItem = new QAction( KIcon("view-list-text"), i18n("View Log..."), this );
	menu->addAction( viewItem );
	connect( viewItem, SIGNAL(triggered(bool)), this, SLOT(slotLogButton()) );
	
	confItem = new QAction( KIcon("configure"), i18n("Configure..."), this );
	menu->addAction( confItem );
	connect( confItem, SIGNAL(triggered(bool)), this, SLOT(slotDialConfig()) );
	menu->addSeparator();
	
	KHelpMenu *helpMenu = new KHelpMenu(this, KGlobal::mainComponent().aboutData(), true);
	QMenu *helpItem = (QMenu*)helpMenu->menu();
	helpItem->setIcon( KIcon("help-contents") );
  menu->addMenu( helpItem );
	dialMenuButton->setMenu( menu );
	
	dialLineEdit = new KLineEdit( dialLocNum );
	dialLineEdit->setClearButtonShown( true );
	dialLineEdit->setClickMessage( i18n("Drag address book entries here") );
	dialLineEdit->setToolTip( i18n("After dragging, select a number from the drop-down") );
	dialLineEdit->setUrlDropsEnabled( true );
	dialLocNum->setLineEdit( dialLineEdit );

	busyTimer = new QTimer( this );
	redialTimer = new QTimer( this );
	  				
  connect( busyTimer, SIGNAL(timeout()), this, SLOT(slotBusyTimeout()) );
  connect( redialTimer, SIGNAL(timeout()), this, SLOT(slotRedialTimeout()) );
	connect( dialLocNum, SIGNAL(currentIndexChanged(int)), this, SLOT(slotIndexChanged(int)) );
	connect( dialLineEdit, SIGNAL(clearButtonClicked()), this, SLOT(slotClearEdit()) );
	
	setStatusBar( i18n("Ready") );
	
	Q_UNUSED(parent);
	Q_UNUSED(fl);
}

DialDialog::~DialDialog()
{
	if ( locked )
		modem->unlockDevice();       						              // release modem lock if held
	if ( modem > 0 )
	{
		delete modem; modem = 0;
	}
}

void DialDialog::slotDialHangup()
{
	if ( modem == 0 )                                           // first entry
	{
		modem = new ModemComms( modemDevice.toLatin1() ); 				// get a modem
		locked = modem->lockDevice();
	}

	if ( !locked )
	{
		KMessageBox::sorry( this, i18n( "Unable to lock serial port.\nModem probably in use." ),
	        	i18n( "Warning" ) );
  	reject();
  }
	else
  {
  	if ( !dialling )
  	{                                       // Dial clicked
  		dialDialButton->setEnabled( false );
			setCursor( Qt::WaitCursor );
  		dialling = dialNumber( dialLocNum->currentText() );
  		if ( dialling )
  		{			
				setStatusBar( i18n("Dialling...") );
  			dialDialButton->setText( i18n( "&Hangup" ) );
  			dialDialButton->setShortcut( Qt::ALT + Qt::Key_H );
  		  dialDialButton->setToolTip( i18n("Hangup the modem") );
  		  modem->setICanon( false );					// don't wait for newlines	
  		  busyTimer->start( 100 );     				// pop the busy checker every second
  			dialDialButton->setEnabled( true );
				dialDialButton->setFocus();
				setCursor( Qt::ArrowCursor );
  		}
  		else
  		{
				resetDialBtn();
				reject();														// problem with serial port
  		}
  	}
  	else
  	{                                       // Hangup clicked
  		busyTimer->stop();
  		hangUp();
			resetDialBtn();
  	  accept();
  	}
	}
}

void DialDialog::slotDialCancel()
{
	if ( intvlLeft > 0 )
		redialTimer->stop();		
	if ( dialling )
	{
		busyTimer->stop();
		hangUp();
	}
	
	reject();
	resetDialBtn();
}

void DialDialog::slotDialConfig()
{
	ConfigDialogImp config( this, 0 );
	if ( config.exec() == QDialog::Accepted )
		kall->saveOptions();                  // save new options
}

void DialDialog::slotLogButton()
{
	LogViewDialogImp *logview = new LogViewDialogImp( this );
	const int w = genGroup->readEntry( "LogSizeW", "0" ).toInt();
	const int h = genGroup->readEntry( "LogSizeH", "0" ).toInt();
  if( w > 0 )
  {
    logview->resize( QSize( w, h ) );
  }
  
  QByteArray cols = genGroup->readEntry( "LogCol-Sizes", QByteArray() );
	if ( !cols.isEmpty() )
		logview->rows->header()->restoreState( cols );

	logview->exec();	

	genGroup->writeEntry( "LogSizeW", logview->size().width() );
	genGroup->writeEntry( "LogSizeH", logview->size().height() );
	genGroup->writeEntry( "LogCol-Sizes", logview->rows->header()->saveState() );
	genGroup->config()->sync();
	delete logview; 
}

void DialDialog::slotDialHelp() 
{
  KToolInvocation::invokeHelp();
}

void DialDialog::keyPressEvent( QKeyEvent *qkeyevent )
{
	int k = qkeyevent->key();
	if ( k == Qt::Key_Escape )
 	{
 		qkeyevent->accept();
 		slotDialCancel();
 	}
	else
		qkeyevent->ignore();
}

bool DialDialog::dialNumber( QString num )
{
 	if ( modem->initPort() == 0 )
 	{
	  if ( !modem->modemIO(modemInit1.toLatin1())  )
	  {
  		KMessageBox::error( this,
      			i18n( "No response from serial port.\nAre you sure there is a modem connected?" ),
      			i18n( "Severe error" ) );
     	modem->closePort();
    	return false;
	  }
   	modem->modemIO( modemInit2.toLatin1() );
   	modem->modemIO( QString( modemDial + num ).toLatin1() );
 	}
 	else
 	{
 		KMessageBox::error( this,
     			i18n("Unable to open serial port ") + modemDevice, i18n("Severe error" ) ) ;
   	return false;
  }
	return true;
}

void DialDialog::hangUp()
{
	setCursor(Qt::WaitCursor);
  modem->setICanon( true );						// wait for newlines	
 	modem->modemIO( QString("+++" + modemHangup).toLatin1() );
 	modem->closePort();
	setCursor(Qt::ArrowCursor);
}

void DialDialog::slotBusyTimeout()
{
  QString resp = QString( modem->getResp() ).trimmed();
	if ( resp == "BUSY" )
	{
		busyTimer->stop();
		dialling = false;
		dialDialButton->setEnabled( false );
	  hangUp();
	  ++redialCount;
    if ( redialCount > maxRetry )
	  {
	  	slotDialCancel();       // simulate user cancel
	  }
	  else
	  {
  	  setStatusBar( i18n("Redial attempt %1 of %2 in %3 seconds")
										.arg(redialCount).arg(maxRetry).arg(intvlRetry) );
  	  redialTimer->start( 1000 );
			intvlLeft = intvlRetry;
	  }
	}
  else if ( resp == "NO DIALTONE" )
	{
		busyTimer->stop();
 		KMessageBox::error( this, i18n("No dial tone - check hand-set on-hook."),
				i18n("Severe error") );
		reject();
	}
}

void DialDialog::slotRedialTimeout()
{
	--intvlLeft;
	if ( intvlLeft == 0 )
	{
		redialTimer->stop();
		setStatusBar( "" );
		kapp->processEvents();
		slotDialHangup();						//	simulate user dial
	}
	else
	{
 	  setStatusBar( i18n("Redial attempt %1 of %2 in %3 seconds")
  	  				.arg(redialCount).arg(maxRetry).arg(intvlLeft) );
	}
}

void DialDialog::setDialInfo( const QString &number )
{
	Akonadi::RecursiveItemFetchJob *job = new Akonadi::RecursiveItemFetchJob( Akonadi::Collection::root() );
	job->fetchScope().fetchFullPayload();
	connect( job, SIGNAL(result(KJob*)), this, SLOT(slotFetchJob(KJob*)) );

	job->start();
	mNumber = number;
}

void DialDialog::slotFetchJob( KJob *job )
{
  Akonadi::RecursiveItemFetchJob *fetchJob = qobject_cast<Akonadi::RecursiveItemFetchJob*>( job );
  const Akonadi::Item::List items = fetchJob->items();
	KABC::Addressee contact;
	bool found = false;
	
	foreach ( const Akonadi::Item &item, items )
	{	
    if ( item.hasPayload<KABC::Addressee>() ) 
		{
			contact = item.payload<KABC::Addressee>();
			PhoneNumber::List phones = contact.phoneNumbers();

			foreach ( const KABC::PhoneNumber &phone, phones )
			{
				if ( phone.number() == mNumber )
				{
					found = true; break;
				}	
			}	
		}
		
		if ( found ) break;
	}
	
	if ( found )
		addNumbers( contact );
	else
	{	
		qDebug() << "Addressee not found";
		localise( mNumber );
		dialLocNum->addItem( mNumber, QStringList() );
	}	
}

void DialDialog::slotSearchJob( KJob *job )
{
	if ( job->error() )
	{
		qDebug() << "Search job error: " << job->errorString();
		return;
	}	
	
	Akonadi::ItemSearchJob *searchJob = qobject_cast<Akonadi::ItemSearchJob*>( job );
  const Akonadi::Item::List items = searchJob->items();
	
	if ( items.count() > 0 )
	{	
		foreach ( const Akonadi::Item &item, items )
		{	
			const KABC::Addressee contact = item.payload<KABC::Addressee>();
			qDebug() << contact.formattedName();
		}
		
		// kludge because more than one item is returned
		const KABC::Addressee contact = items[1].payload<KABC::Addressee>();
		addNumbers( contact );
	}
	else
	{	
		qDebug() << "Addressee not found";
		localise( mNumber );
		dialLocNum->addItem( mNumber, QStringList() );
	}	
}

void DialDialog::getDialInfo( QString &person, QString &number, QString &org )
{
	number = dialLocNum->currentText();
	QStringList info( dialLocNum->itemData( dialLocNum->currentIndex() ).toStringList() );
	
	if ( !info.empty() )
	{
		person = info[0]; org = info[1];
	}
}

void DialDialog::slotEditChanged( const QString &vcf )
{
	KUrl url( vcf );

	if ( url.scheme() == "file" )
	{
		QFile file( url.toLocalFile() );
		dialLocNum->clear();
		
		if ( file.open( QIODevice::ReadOnly ) )
		{
			const QByteArray data = file.readAll();
			file.close();

			KABC::VCardConverter converter;
			const KABC::Addressee::List contacts = converter.parseVCards( data );

			mNumber.clear();
			foreach ( const KABC::Addressee &contact, contacts )
			{
				addNumbers( contact );
			}
		}
	}
	else if ( url.scheme() == "akonadi" )
	{
		Akonadi::Item item = Akonadi::Item::fromUrl( url );
		Akonadi::ItemFetchJob *job = new Akonadi::ItemFetchJob( item );

		dialLocNum->clear();
		job->fetchScope().fetchFullPayload();
		connect( job, SIGNAL(result(KJob*)), this, SLOT(slotFetchedItem(KJob*)) );	
		mNumber.clear();
	}
}

void DialDialog::slotFetchedItem( KJob *job )
{
	if ( job->error() ) 
	{
		qDebug() << "Error occurred in fetchjob";
		return;
	}

	Akonadi::ItemFetchJob *fetchJob = qobject_cast<Akonadi::ItemFetchJob*>( job );
  const Akonadi::Item::List items = fetchJob->items();
	
  foreach( const Akonadi::Item &item, items ) 
	{
    if ( item.hasPayload<KABC::Addressee>() ) 
		{
			const KABC::Addressee contact = item.payload<KABC::Addressee>();
			addNumbers( contact ); 
		}
  }
}

void DialDialog::addNumbers( const KABC::Addressee &contact )
{
	if ( !mNumber.isEmpty() )
	{
		QString number( mNumber );				
		localise( number );
		dialLocNum->addItem( number, QStringList() << contact.formattedName() << contact.organization() );
	}	
	else
	{	
		PhoneNumber::List phones = contact.phoneNumbers();

		foreach ( const KABC::PhoneNumber &phone, phones )
		{
			if ( phone.type() != PhoneNumber::Fax )
			{
				QString number( phone.number() );				
				localise( number );
				dialLocNum->addItem( number, QStringList() << contact.formattedName() << contact.organization() );
			}
		}
	}
	
	if ( dialLocNum->count() > 0 )
		slotIndexChanged( 0 );
}

void DialDialog::slotIndexChanged( int k )
{
	QString number( dialLocNum->itemText( k ) );
	if ( !number.isEmpty() )
	{
		QStringList info( dialLocNum->itemData( k ).toStringList() );
		
		if ( !info.empty() )
		{
			dialLineEdit->setToolTip( "<pre><b>Person:</b>       " + info[0] + 
																"<br><b>Number:</b>       " + number +
																"<br><b>Organisation:</b> " + info[1] +
																"</pre>");
		}
	}
	else
		dialLineEdit->setToolTip( i18n("After dragging, select a number from the drop-down") );
}

void DialDialog::slotClearEdit()
{
	dialLineEdit->clear();
	dialLocNum->clear();
}

void DialDialog::resetDialBtn()
{
	dialDialButton->setText( i18n( "&Dial" ) );
  dialDialButton->setShortcut( Qt::ALT + Qt::Key_D );
  dialDialButton->setToolTip( i18n("Dial number") );
}

void DialDialog::setStatusBar( const QString &text )
{
	dialText->setText( text );
}


// Code to perform Akonadi/Nepomuk search which doesn't work very well
/*	const QString query = QString( "SELECT ?person WHERE "
	"{ "
		"?person <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#hasPhoneNumber> " 
		"?number . "
		"?number <http://www.semanticdesktop.org/ontologies/2007/03/22/nco#phoneNumber> "
				"\"%1\"^^<http://www.w3.org/2001/XMLSchema#string> . "
	"}" ).arg(number); 
	
	Akonadi::ItemSearchJob *job = new Akonadi::ItemSearchJob( query );
	job->fetchScope().fetchFullPayload();
	connect( job, SIGNAL(result(KJob*)), this, SLOT(slotSearchJob(KJob*)) );*/

